@@ -34,8 +34,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#include "glibcompat.h" /* for g_list_free_full */
-
 typedef enum
 {
     CAJA_QUERY_EDITOR_ROW_LOCATION,
@@ -92,7 +90,7 @@ enum
     LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL];
+static guint signals[LAST_SIGNAL] = { 0 };
 
 static void  caja_query_editor_class_init       (CajaQueryEditorClass *class);
 static void  caja_query_editor_init             (CajaQueryEditor      *editor);
